/***********************************************************************
 * Add the suitable linker script (e.g., p30f4011.gld) to the project. *
 *                                                                     *
 * If interrupts are not used, all code presented for that interrupt   *
 * can be removed or commented out with C-style comment declarations.  *
 *                                                                     *
 * For additional information about dsPIC architecture and language    *
 * tools, refer to the following documents:                            *
 *                                                                     *
 * MPLAB C30 Compiler User's Guide                        : DS51284    *
 * dsPIC 30F MPLAB ASM30, MPLAB LINK30 and Utilites                    *
 *                                           User's Guide : DS51317    *
 * Getting Started with dsPIC DSC Language Tools          : DS51316    *
 * dsPIC 30F Language Tools Quick Reference Card          : DS51322    *
 * dsPIC 30F 16-bit MCU Family Reference Manual           : DS70046    *
 * dsPIC 30F Motor Control and Power Conversion Family                 * 
 *                                             Data Sheet : DS70082    *
 * dsPIC 30F Programmer's Reference Manual                : DS70030    *
 *                                                                     *
 * Template file has been compiled with MPLAB C30 v1.30.               *
 *                                                                     *
 ***********************************************************************
 *                                                                     * 
 *    Author:            DDS                                           *
 *    Company:           University of Pisa                            * 
 *    File Description:  Motor DC Identification                       *
 *    Date:              -/-/2007                                      *
 *    File Version:      1.0                                           *
 *    Other Files Required: p33FJ256MC710.gld                          *
 *                			                                           *
 *                                                                     *
 ***********************************************************************
 *                                                                     *
 * Other Comments:                                                     *
 *                                                                     *
 * 1) C attributes, designated by the __attribute__ keyword, provide a *
 *    means to specify various characteristics of a variable or        *
 *    function, such as where a particular variable should be placed   *
 *    in memory, whether the variable should be aligned to a certain   *
 *    address boundary, whether a function is an Interrupt Service     *
 *    Routine (ISR), etc. If no special characteristics need to be     *
 *    specified for a variable or function, then attributes are not    *
 *    required. For more information about attributes, refer to the    *
 *    C30 User's Guide.                                                *
 *                                                                     *
 * 2) The __space__(xmemory) and __space__(ymemory) attributes are     *
 *    used to place a variable in X data space and Y data space,       *
 *    respectively. Variables accessed by dual-source DSP instructions *
 *    must be defined using these attributes.                          *
 *                                                                     *
 * 3) The aligned(k) attribute, used in variable definitions, is used  *
 *    to align a variable to the nearest higher 'k'-byte address       * 
 *    boundary. 'k' must be substituted with a suitable constant       *
 *    number when the ModBuf_X(k) or ModBuf_Y(k) macro is invoked.     *
 *    In most cases, variables are aligned either to avoid potential   *
 *    misaligned memory accesses, or to configure a modulo buffer.     *                           
 *                                                                     *
 * 4) The __interrupt__ attribute is used to qualify a function as an  *
 *    interrupt service routine. An interrupt routine can be further   *
 *    configured to save certain variables on the stack, using the     *
 *    __save__(var-list) directive.                                    *
 *                                                                     *
 * 5) The __shadow__ attribute is used to set up any function to       *  
 *    perform a fast context save using shadow registers.              *
 *                                                                     *
 * 6) Note the use of double-underscores (__) at the start and end of  *
 *    all the keywords mentioned above.                                *
 *                                                                     *
 **********************************************************************/      

#include "p33FJ256MC710.h"
#include <math.h>
#include "pwm.h"
#include "uart.h"
#include "adc.h"
#include "timer.h"   
                                                                              
_FOSCSEL(  FNOSC_PRIPLL );                                                                          
_FOSC( FCKSM_CSDCMD  & OSCIOFNC_OFF & POSCMD_XT );
_FWDT( FWDTEN_OFF );
_FGS( GSS_OFF & GCP_OFF & GWRP_OFF);               /* Disable Code Protection */

/* ************ Define constants here ************* */

#define dim_data 13000 	// -> n_prbs*periods*camp
//#define dim_vin 13000 	// -> n_prbs*periods*camp
#define dim_p 500    	// -> n_prbs*periods/8 
#define dim_dc 2000   	// -> n_prbs*periods

/************* START OF GLOBAL DEFINITIONS **********/
// Tcy = 8e-8 sec;

unsigned int DC = 159;        	    	/* PWM -> prbs signal or pwm&prbs (es 999 o 4499 o 22499)*/ 
unsigned long int Tsample = 7;			/* Tempo di campionamento -> Timer23 (4.096e-5 sec) Prescaler -> 1:64*/
unsigned int c_lim = 500;              	/* Current limit (with scale-factor 2)...(697/2->2.25A infact 2.25/3.3*1023/2)*/
unsigned char Buf[4];              		/* UART1 Rx Buffer */ 
char st[4];								/* UART1 Tx Buffer */
unsigned int icount = 0, pcount = 0, dc_count = 0, pwmc = 0, t4count = 1;
unsigned int switch_prbs = 0, dind = 1, camp = 20, tad = 5, switch_adc = 0;
unsigned long int Tpwm = 0;				/* pre = 64; ATT!!! ...like timer23 prescaler... */
unsigned int *dcv;
unsigned char *iav, *psv, *vav;

/************** END OF GLOBAL DEFINITIONS ***********/


void ADC1_config(void);

void DC_config();

void PWM_config(unsigned int dc, unsigned int Ts);

void UART1_config(void);

void TIMER4_config(unsigned long int match_value);

void TIMER23_config(unsigned long int match_value);

void send_by_UART1(unsigned int *buffer, unsigned int n);

void __attribute__((__interrupt__)) _ADC1Interrupt(void);

void __attribute__((__interrupt__)) _U1RXInterrupt(void);

void __attribute__((__interrupt__)) _T5Interrupt(void);

void send_PRBS(unsigned int num);

void send_DCV(unsigned int num);

void reset_counters();

void stop_simulation();


/************* START OF MAIN FUNCTION ***************/

int main ( void )
{ 
  	unsigned char ias[dim_data]; 
	unsigned int dcs[dim_dc];		// -> Use dcv, iav, psv because global pointers...
	unsigned char vas[dim_data];
	iav = ias;
	dcv = dcs;
	vav = vas;
	
	unsigned char prbs[dim_p]={
	0x81, 0x6A, 0x46, 0xB4,
    0x9C, 0x90, 0x92, 0xED,
    0x78, 0xC1, 0x8A, 0xE9,
    0x86, 0xEB, 0xF9, 0x7E,
    0x40, 0x35, 0x23, 0x5A,
    0x4E, 0x48, 0xC9, 0x76,
    0xBC, 0x60, 0xC5, 0x74,
    0xC3, 0xF5, 0x7C, 0x3F,
    0xA0, 0x9A, 0x11, 0x2D,
    0x27, 0xA4, 0x64, 0x3B,
    0x5E, 0xB0, 0x62, 0xBA,
    0xE1, 0x7A, 0xBE, 0x1F,
    0x50, 0xCD, 0x88, 0x96,
    0x13, 0x52, 0xB2, 0x1D,
    0x2F, 0x58, 0x31, 0xDD,
    0x70, 0x3D, 0xDF, 0x0F,
    0xA8, 0x66, 0x44, 0xCB,
    0x09, 0x29, 0xD9, 0x8E,
    0x17, 0xAC, 0x98, 0x6E,
    0xB8, 0x9E, 0xEF, 0x07,
    0x54, 0x33, 0xA2, 0xE5,
    0x84, 0x94, 0x6C, 0xC7,
    0x0B, 0x56, 0x4C, 0x37,
    0x5C, 0xCF, 0xF7, 0x03,
    0xAA, 0x19, 0xD1, 0x72,
    0x42, 0x4A, 0xB6, 0xE3,
    0x05, 0x2B, 0xA6, 0x1B,
    0xAE, 0xE7, 0xFB, 0x01,
    0xD5, 0x8C, 0x68, 0x39,
    0x21, 0x25, 0xDB, 0xF1,
    0x82, 0x15, 0xD3, 0x0D,
    0xD7, 0xF3, 0xFD
	};

	pcount = 80;		

	UART1_config();         // Initialize the UART module to receive data.

  	TRISEbits.TRISE2 = 0;   // Enable signal on RE2. 

  	psv = prbs;

	PORTEbits.RE2 = 0;      // H-Bridge disabled.	

    st[0] = 0xAA;			// Start Signal.
  	st[1] = 0xBB;
  	st[2] = 0xCC;
  	st[3] = '\0'; 
  	send_by_UART1((unsigned int *)st,3); // Characters transimitted in hex.
  
  	while(1)
    	if(dc_count!=0 && dind>(dc_count+1))
      		stop_simulation();                 
  	return 0;
}


/****** START OF INTERRUPT SERVICE ROUTINES *********/
/* Replace the interrupt function names with the    */
/* appropriate names depending on interrupt source. */
/* The names of various interrupt functions for     */
/* each device are defined in the linker script.    */
/* Interrupt Service Routine 1                      */
/* No fast context save, and no variables stacked   */
/****************************************************/

/* This is: T4 ISR */
void __attribute__((__interrupt__)) _T4Interrupt(void){
	
	IFS1bits.T4IF = 0;
	if(t4count == 1000){ 
		T2CONbits.TON= 1;		// Turn on Timer23. 
		AD1CON1bits.ADON = 1; 	// Turn on ADC1
		CloseTimer4();       	// Turn off Timer4. 
	}
	if(t4count == 999) 
		*(&PDC1+2) = dcv[0]; 	// Set DC...
	t4count++;
	
}

/* This is: ADC1 ISR */
void __attribute__((__interrupt__)) _ADC1Interrupt(void) 
{
	/*  Clear the A/D Interrupt flag bit or else the CPU will
        keep vectoring back to the ISR*/
    IFS0bits.AD1IF = 0;
	// Current (output):
	if(switch_adc == 0){
		iav[icount] = ADC1BUF0/2; 		 		// ATT!!! scale-factor = 2...
	//	iav[icount] = 0; 
		if(ADC1BUF0 > c_lim) stop_simulation();  
	}
	if(switch_adc == 1){
		vav[icount] = ADC1BUF0/2; 		// ATT!!! scale-factor = 2...
		if( ADC1BUF0 > c_lim ) stop_simulation();
		/*
		if( pwmc == 0 ){
			*(&PDC1+2) = dcv[dind%dc_count]; 	// Set DC...
			dind++;
		}	
		pwmc = (pwmc+1)%camp;
		icount++;
		*/
	}	
	switch_adc = (switch_adc==0)?1:0;
}

/* This is: UART1 receive ISR */
void __attribute__((__interrupt__)) _U1RXInterrupt(void)
{
    unsigned long int x,y,z;
	IFS0bits.U1RXIF = 0;
    int i=0;
    for(i=0; i<4 && DataRdyUART1();i++)
        Buf[i] = ReadUART1();  
    /* Prompt: AA xx xx xx -> Turn on the real simulation and set the sample-time;
               BB 00 00 00 -> Turn off the real simulation;
               CC .. .. .. -> Set switch_prbs flag to accept prbs data;
               DD 00 xx xx -> Send DutyCycle vector;
               EE .. .. .. -> Send the received prbs signal;
               FF xx xx xx -> New Duty Cycle for the PWM Module; */
 
    if(switch_prbs == 0){
     	switch (Buf[0])
     	{
			/*******************************/
    		case 0xAA: 
			if(PORTEbits.RE2 != 1){
				reset_counters();
            	ADC1_config();					// Configure the A/D converter.
 
			    x = Buf[1];
				y = Buf[2];
				z = Buf[3];
				x = (x<<16) + (y<<8) + z;
                if(x!=0)
                	TIMER23_config(x);       // Timer23(32 bit) configuration.
				else 
					TIMER23_config(Tsample);

				Tpwm = camp*(Tsample+1)-1;	// Calculate the PWM period (0.256sec = Tsample(sec)*camp).
				if(dc_count == 0) DC_config();
				PWM_config(159,Tpwm);	// Configure the PWM module.
				TIMER4_config(Tpwm);	// Configure the Timer4 module.
                st[0] = (x>>16) & 0x00FF;
                st[1] = (x>>8) & 0x00FF;
				st[2] = x & 0x00FF;
                send_by_UART1((unsigned int *)st,3);
                while(BusyUART1());
				T4CONbits.TON= 1;		// Turn on Timer4.
				PORTEbits.RE2 = 1;    	// H-Bridge enabled.
				PTCONbits.PTEN= 1;    	// Turn on PWM.
				// AD1CON1bits.ADON = 1; 	// Turn on ADC1
				// T2CONbits.TON= 1;		// Turn on Timer23. 
				 
			}break;
            /*******************************/                 
			case 0xBB: 
			stop_simulation();
            break;
			/*******************************/
        	case 0xCC: 
			switch_prbs = 1; // Configure the module to receive data (Identification PRBS Input).
            pcount = 0;
			break;  
      		/*******************************/
			case 0xDD:
			DC_config();
			x = Buf[1];
			y = Buf[2];
			z = Buf[3];
			x = (x<<16) + (y<<8) + z;
            if(x!=0){ 
            	x = (x>dc_count)? dc_count: x;   		// Control of the value.
                send_DCV((unsigned int)x);     // PRBS development configuration.
            }
			else send_DCV(dc_count); 			
			break;
			/*******************************/	
			case 0xEE:
			x = Buf[1];
			y = Buf[2];
			z = Buf[3];
			x = (x<<16) + (y<<8) + z;
            if(x!=0){ 
            	x = (x>pcount)? pcount: x;   		// Control of the value.
                send_PRBS((unsigned int)x);     // PRBS development configuration.
            }
			else send_PRBS(pcount); 			
			break;
			/*******************************/
			case 0xFF: // Duty Cycle settings.
			x = (Buf[2]<<8) + Buf[3];
            if(x > 0)  
            	DC = (unsigned int)x;
            if(AD1CON1bits.ADON == 0){
            	st[0] = (char) ((DC>>8) & 0x00FF);
                st[1] = (char) (DC & 0x00FF);
                st[2] = '\0';
                send_by_UART1((unsigned int *)st,2);
                while(BusyUART1());
            }
            break;
			/*******************************/
        	default:     
			if(AD1CON1bits.ADON == 0){
            	char s[5];
                s[0] = (char) 0xFF;
                s[1] = (char) 0xFF;
                s[2] = (char) 0xFF;
                s[3] = (char) 0xFF;
                s[4] = '\0';
                send_by_UART1((unsigned int *)s,4);
                while(BusyUART1());
            }
			/*******************************/
	 	}
	}
	else
	if(switch_prbs == 1){		// Record the PRBS voltage input sequence.
		if( Buf[0]==Buf[1] && Buf[0]==Buf[2] && Buf[0]==Buf[3] ){
			switch_prbs = 0;
		}
		else{
			for(i=0; i<4; i++){		
				psv[pcount] = Buf[i]; 
				pcount++;
			}
		}	
	}       
}

/****************************** User Functions ********************************/
/* Create DC vector */
void DC_config(){
	int i,k;
	dc_count = 0; 
	for(i=0; i<pcount; i++)
		for(k=0; k<8; k++){	
			dcv[dc_count] = ((psv[i]>>k) & 0x0001)*DC;
			dc_count++;
		}		
}

/* Send PRBS signal. */
void send_PRBS(unsigned int num)
{
	int i=0;
	while(i<num){
		st[0] = (char)(psv[i] & 0x00FF);
		i++;
    	send_by_UART1((unsigned int *)st,1);
		while(BusyUART1());
	}
	return;
}

void send_DCV(unsigned int num)
{
	int i=0;
	while(i<num){
		st[0] = (char) ((dcv[i]>>8) & 0x00FF);
        st[1] = (char) (dcv[i] & 0x00FF);
		i++;
    	send_by_UART1((unsigned int *)st,2);
		while(BusyUART1());
	}
	return;
}

/* Stop the simulation. */
void stop_simulation()
{ 
	int i,k;
	CloseADC1();          	// Turn off ADC1.
    PORTEbits.RE2 = 0;    	// H-Bridge disabled. 
    CloseMCPWM();         	// Turn on PWM.
    CloseTimer23();       	// Turn off Timer23. 
    CloseTimer4();       	// Turn off Timer4. 
    while(BusyUART1());

   	// Final results.
    st[0] = (char) 0xEE;
    st[1] = (char) 0xEE;
    st[2] = (char) 0xEE;
    st[3] = (char) 0xEE;
    send_by_UART1((unsigned int *)st,4);
    while(BusyUART1());
    st[0] = (char) 0xDD;
    st[1] = (char) 0xDD;
    st[2] = (char) ((icount>>8) & 0x00FF);
    st[3] = (char) (icount & 0x00FF);
    send_by_UART1((unsigned int *)st,4);
    while(BusyUART1());
	st[0] = (char) 0xDD;
    st[1] = (char) 0xDD;
    st[2] = (char) ((dc_count>>8) & 0x00FF);
    st[3] = (char) (dc_count & 0x00FF);
    send_by_UART1((unsigned int *)st,4);
    while(BusyUART1());
	st[0] = (char) 0xDD;
    st[1] = (char) 0xDD;
    st[2] = (char) ((dind>>8) & 0x00FF);
    st[3] = (char) (dind & 0x00FF);
    send_by_UART1((unsigned int *)st,4);
    while(BusyUART1());
	
	// Border between the vectors (EE EE EE EE EE EE EE EE):
    st[0] = (char) 0xEE;
    st[1] = (char) 0xEE;
    st[2] = (char) 0xEE;
    st[3] = (char) 0xEE;
    send_by_UART1((unsigned int *)st,4);
	while(BusyUART1());
    send_by_UART1((unsigned int *)st,4);

	// Ia:
	for(i=0; i<icount; i++){
	    while(BusyUART1());
   		// Final Ia vector:
		for(k=0;k<1000;k++);
    	st[0] = (char) 0xFF;
    	st[1] = (char) (iav[i] & 0x00FF);
    	send_by_UART1((unsigned int *)st,2);
	}
	while(BusyUART1());
	
	// Border between the vectors (EE EE EE EE EE EE EE EE):
    st[0] = (char) 0xEE;
    st[1] = (char) 0xEE;
    st[2] = (char) 0xEE;
    st[3] = (char) 0xEE;
    send_by_UART1((unsigned int *)st,4);
	while(BusyUART1());
    send_by_UART1((unsigned int *)st,4);

	// Va:
	for(i=0; i<icount; i++){
	    while(BusyUART1());
   		// Final Va vector:
		for(k=0;k<1000;k++);
    	st[0] = (char) 0xFF;
    	st[1] = (char) (vav[i] & 0x00FF);
    	send_by_UART1((unsigned int *)st,2);
	}
	while(BusyUART1());

	// The end:
	st[0] = (char) 0xEE;
    st[1] = (char) 0xEE;
    st[2] = (char) 0xEE;
    st[3] = (char) 0xEE;
    send_by_UART1((unsigned int *)st,4);
	while(BusyUART1());
    send_by_UART1((unsigned int *)st,4);
	while(BusyUART1());

	reset_counters();
    return;     
}

void reset_counters(){
	icount = 0;
	t4count = 1;
    dc_count = 0;
	dind = 1;
	pwmc = 0;
	switch_prbs = 0;
	switch_adc = 0;
	return;
}







